; Multiple effects in 64 bytes, with transitions and basic sound, no precalc.
; Based on variation of the "plasma effect".
;
; How it works?
; - There is a "sine generator" - no sine/cosine on 6502 (nor easilly accessible in Atari XE/XL OS) so had to be code-generated.
; - The generated "sine" is iteratively modified by OS timer (xor) for different patterns
; - In the same loop character set and colors are set
; - Real time transition between effects is assured by real-time calc in each frame, with desired state of registers.
; - Reverse engineering OS of Atari I found very useful code snippet (16bit substract at $F581) very useful for full-screen effects.
; - Sound is an output of OS timer modified by register state in drawing procedure
; - There are many configs for different hardware/software settings + one "safe" configuration that doesn't rely on known initial state of machine

CONFIG_EMU = 0
CONFIG_BWDOS = 1
CONFIG_SIO2SD = 2
CONFIG_SDRIVE = 3
CONFIG_SAFE = 4
CONFIG = CONFIG_EMU
;CONFIG = CONFIG_BWDOS
;CONFIG = CONFIG_SIO2SD
;CONFIG = CONFIG_SDRIVE
;CONFIG = CONFIG_SAFE

;RANDOM	equ $d20a
;WSYNC	equ $D40a
;VCOUNT	equ $D40B

;;;;;; SOUND 
AUDF1	equ $D200
AUDCTL	equ $D208
SKCTL   equ $D20F

;;;;;; GFX
GPRIOR  equ $026F

; OS functions
TIMER1  equ $13
TIMER2  equ $14
ADRESS equ $64

SUBSTRACT equ $F581
SUBTMP EQU $029E
openmode  equ $ef9c
RAMTOP equ $6A

Y_VALUE equ $87
sine equ ($100-Y_VALUE)
temp equ sine-1

	org $2000
			
start:
.if CONFIG = CONFIG_EMU
	; TIMERS are 0
.elif CONFIG = CONFIG_BWDOS
	stx TIMER1 ; X=0 in BW-DOS
.elif CONFIG = CONFIG_SIO2SD 
	sty TIMER1 ; Y=$FF in SIO2SD loader
.elif CONFIG = CONFIG_SDRIVE
	sty TIMER1 ; Y=0 in SDRIVE loader
.elif CONFIG = CONFIG_SAFE ; SUPER SAFE CONFIGURATION TRYING TO ADDRESS ALL OTHER POSSIBLE LOADERS
	ldx #$0
	stx TIMER1
	stx TIMER2
	stx AUDCTL
	lda #3
	sta SKCTL
	lda #$C0
	sta RAMTOP	
	lda #0
	jsr openmode
.endif
NEXT_FRAME:
	; X=FF, Y=0
	 txa
	 ; a=ff
	 ldy #Y_VALUE


	; Sine function is based on very cool piece of code:
	; http://forum.6502.org/viewtopic.php?f=2&t=2404
	; We control in this loop also colors, charset and pattern	 
loop1:
	 dex
loop2:
	 sta sine-1,y
	 eor TIMER1 ; for flood of effects
	 sta GPRIOR+21,y ; modify at the same time colors and charset
	 dey
	 beq exit_loops
	 stx temp
	 adc temp 
	 bmi loop1
	 inx
	 jmp loop2
exit_loops:

	; y=0
	dey
	sty ADRESS 
	lda #$BF
	sta ADRESS+1 

	ldx #23 ; row
NEXT_ROW:
	ldy #40 ; column
	sty SUBTMP
	jsr substract ; return oriented programming ;)
ROW_LOOP:
	lda TIMER2
	sta AUDF1-20,x 
	adc sine,y
	adc sine,x
	sta (adress),y
	dey
	bne ROW_LOOP		
	dex	
	bpl NEXT_ROW		
	bmi NEXT_FRAME
	
.if CONFIG = CONFIG_SAFE
	RUN start
.endif